/********************************************************************/
/* Inflation, torsion and axial extension of a homogeneous          */
/* Mooney-Rivlin cylindrical tube, based on the equations of        */
/* R. S. Rivlin (1950), 'Large Elastic Deformations of Isotropic    */
/* Materials VI. Further Results in the Theory of Torsion, Shear    */
/* and Flexure,' Philosophical Transactions, vol. A242, pp 173-195. */
/*                                                                  */
/* Written by Martyn Nash Jan-1997                                  */
/********************************************************************/

/* -- Include Files -- */
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>
#include <cgi-decode.h>
/* For: CGI handling functions */


#define CENTI 1.0
#define KILO 1.0
/*
#define CENTI 0.01
#define KILO 1000.0
*/
#define EXAMPLE_NUM "537"
#define EXAMPLE_PATH "5/53/"EXAMPLE_NUM

#define PI 3.14159265358979323846264

#define CGI_HEADER \
  "Content-type: text/html \n\n" \
  "<HTML><HEAD>" \
  "<TITLE>analytic solution to an advection diffusion problem of heat flow in glass bath</TITLE>" \
  "</HEAD><BODY>" \
  "<H1>Inflation, axial extension and torsion of an isotropic, homogeneous cylinder - analytic solution</H1>" \
  "<P> This form calculates the analytic solution for the inflation, axial extension and torsion of a cylindrical tube of length d with external and internal radii, a1 and a2 respectively. Material is isotropic homogeneous with W = c1*(I1-3) + c2*(I2-3), where c1 and c2 are the material constants. Undeformed coordinates (R,T,Z) are deformed into (r,t,z). The tube is strained by:" \
  "<UL>" \
  "  <LI> a uniform axial extn, L" \
  "  <LI> a uniform inflation where external and internal radii change to r1 = u1*a1 and r2 = u2*a2 respectively" \
  "  <LI> a uniform simple torsion with the angle of rotation of V between opposite ends of the cylinder. ie) torsion is proportional to distance from one end (proportionality constant S = V/d)." \
  "</UL>" \
  "The deformation described by:" \
  "<UL>" \
  "  <LI> r = sqrt{r1*r1 - (a1*a1 - R*R)/L}" \
  "  <LI> t = T + S*Z" \
  "  <LI> z = L*Z" \
  "</UL>" \
  "with boundary conditions:" \
  "<UL>" \
  "  <LI> applied pressure, pa (kPa) on the internal face" \
  "  <LI> applied pressure, pb (kPa) on the external face" \
  "</UL></P>" \
  "The equations were taken from R. S. Rivlin (1950), 'Large Elastic Deformations of Isotropic Materials VI. Further Results in the Theory of Torsion, Shear and Flexure,' Philosophical Transactions, vol. A242, pp 173-195." \
  "<HR>" \
  "\n"

#define CGI_FORM \
  "<P><FORM METHOD=\"GET\" " \
  "ACTION=\""CMISS_WWW_URL"scripts/analytic_"EXAMPLE_NUM".cgi\">" \
  "External radius a1 (cm): <input name=\"a1\" size=10 value=\"1.5\"> <br>" \
  "Internal radius a2 (cm): <input name=\"a2\" size=10 value=\"1.0\"> <br>" \
  "Cylinder length, d (cm): <input name=\"d\" size=10 value=\"1.0\"> <br>" \
  "Axial extn ratio, L : <input name=\"L\" size=10 value=\"1.2\"> <br>" \
  "Torsion angle, V (degrees): <input name=\"V\" size=10 value=\"30.0\"> <br>" \
  "Material constant, c1 (kPa): <input name=\"c1\" size=10 value=\"2.0\"> <br>" \
  "Material constant, c2 (kPa): <input name=\"c2\" size=10 value=\"6.0\"> <br>" \
  "External applied pressure, pa (kPa): <input name=\"pa\" size=10 value=\"0.0\"> <br>" \
  "Internal applied pressure, pb (kPa): <input name=\"pb\" size=10 value=\"1.5\"> <br>" \
  "Undeformed radius at which to calc soln, Rs (cm): <input name=\"Rs\" size=10 value=\"1.279\"> <br>" \
  "<input type=\"submit\" value=\"Calculate Analytic\">" \
  "<input type=\"reset\" value=\"Reset defaults\">" \
  "</FORM></P><HR>" 

#define CGI_FOOTER \
  "<A HREF=\""CMISS_WWW_URL"help/examples/example_files/"EXAMPLE_PATH"/generated/test_page.html\">Return to the example page</A>"

void GetValues(
  CGI *cgi,
  double *a1,
  double *a2,
  double *d,
  double *L,
  double *V,
  double *S,
  double *c1,
  double *c2,
  double *pa,
  double *pb,
  double *Rs)
{
  char *a1str, *a2str, *dstr, *Lstr, *Vstr, *c1str, *c2str, *pastr, 
    *pbstr, *Rsstr;
  double a1_nonSI, a2_nonSI, d_nonSI, L_nonSI, V_nonSI, c1_nonSI, c2_nonSI, 
    pa_nonSI, pb_nonSI, Rs_nonSI;
    
  if( a1str = lookupString( cgi, "a1" ) )
  {
    a1_nonSI = atof( a1str );
  }
  else  
  {
    a1_nonSI = 1.5;
  }
  if( a2str = lookupString( cgi, "a2" ) )
  {
    a2_nonSI = atof( a2str );
  }
  else  
  {
    a2_nonSI = 1.0;
  }
  if( dstr = lookupString( cgi, "d" ) )
  {
    d_nonSI = atof( dstr );
  }
  else  
  {
    d_nonSI = 1.0;
  }
  if( Lstr = lookupString( cgi, "L" ) )
  {
    L_nonSI = atof( Lstr );
  }
  else  
  {
    L_nonSI = 1.2;
  }
  if( Vstr = lookupString( cgi, "V" ) )
  {
    V_nonSI = atof( Vstr );
  }
  else  
  {
    V_nonSI = 30.0;
  }
  if( c1str = lookupString( cgi, "c1" ) )
  {
    c1_nonSI = atof( c1str );
  }
  else  
  {
    c1_nonSI = 2.0;
  }
  if( c2str = lookupString( cgi, "c2" ) )
  {
    c2_nonSI = atof( c2str );
  }
  else  
  {
    c2_nonSI = 6.0;
  }
  if( pastr = lookupString( cgi, "pa" ) )
  {
    pa_nonSI = atof( pastr );
  }
  else  
  {
    pa_nonSI = 0.0;
  }
  if( pbstr = lookupString( cgi, "pb" ) )
  {
    pb_nonSI = atof( pbstr );
  }
  else  
  {
    pb_nonSI = 1.5;
  }
  if( Rsstr = lookupString( cgi, "Rs" ) )
  {
    Rs_nonSI = atof( Rsstr );
  }
  else  
  {
    Rs_nonSI = 1.279;
  }
  *a1 = a1_nonSI * CENTI;
  *a2 = a2_nonSI * CENTI;
  *d = d_nonSI * CENTI;
  *L = L_nonSI;
  *V = V_nonSI * PI / 180.0;
  *S = ( *V ) / ( *d );
  *c1 = c1_nonSI * KILO;
  *c2 = c2_nonSI * KILO;
  *pa = pa_nonSI * KILO;
  *pb = pb_nonSI * KILO;
  *Rs = Rs_nonSI * CENTI;
}

int CheckValues(
  double a1,
  double a2,
  double d,
  double L,
  double V,
  double c1,
  double c2,
  double pa,
  double pb,
  double Rs)
{
  int error=0;

  if( a1 <= 0.0 )
  {
    error = 1;
    printf( "ERROR: a1 must be positive <BR> \n<HR>\n" );
  }
  if( a2 <= 0.0 || a2 >= a1 )
  {
    error = 1;
    printf( "ERROR: a2 must be positive and less than a1 <BR> \n<HR>\n" );
  }
  if( d <= 0.0 )
  {
    error = 1;
    printf( "ERROR: d must be positive <BR> \n<HR>\n" );
  }
  if( L <= 0.0 )
  {
    error = 1;
    printf( "ERROR: L must be positive <BR> \n<HR>\n" );
  }
  if( ( V < - PI ) || ( V > PI ) )
  {
    error = 1;
    printf( "ERROR: V must be in the range [-180,180] <BR> \n<HR>\n" );
  }
  /*if( pa < 0.0 )
  {
    error = 1;
    printf( "ERROR: pa must be non-negative <BR> \n<HR>\n" );
  }
  if( pb < 0.0 )
  {
    error = 1;
    printf( "ERROR: pb must be non-negative <BR> \n<HR>\n" );
  }*/
  if( Rs <= a2 || Rs >= a1 )
  {
    error = 1;
    printf( "ERROR:  Rs outside the range %7.2f -- %7.2f <BR> \n<HR>\n", 
	    a2, a1 );
  }
  return error;
}

void PrintValues(
  double a1,
  double a2,
  double d,
  double L,
  double V,
  double c1,
  double c2,
  double pa,
  double pb,
  double Rs)
{
  printf( "Analytic solution for :\n" );
  printf( "<UL>\n" );
  printf( "<LI> External radius, a1 = %7.2f cm\n", a1 / CENTI );
  printf( "<LI> Internal radius, a2 = %7.2f cm\n", a2 / CENTI );
  printf( "<LI> Cylinder length, d = %7.2f cm\n", d / CENTI );
  printf( "<LI> Axial extn ratio, L = %7.2f\n", L );
  printf( "<LI> Torsion angle, V = %7.2f degrees\n", V * 180.0 / PI );
  printf( "<LI> Material constant, c1 = %7.2f kPa\n", c1 / KILO );
  printf( "<LI> Material constant, c2 = %7.2f kPa\n", c2 / KILO );
  printf( "<LI> External pressure, pa = %7.2f kPa\n", pa / KILO );
  printf( "<LI> Internal pressure, pb = %7.2f kPa\n", pb / KILO );
  printf( "<LI> Undeformed radius for solution calculations, Rs = %10.4f cm\n", 
	  Rs / CENTI );
  printf( "</UL>\n" );
  printf( "<HR>\n" );
}

double K_u1_calc(
  double a1,
  double L,
  double u1)
{
  double K_temp;

  K_temp = a1 * a1 * ( L * u1 * u1 - 1.0 );

  return K_temp;
}

double u2_calc(
  double a1,
  double a2,
  double L,
  double u1)
{
  double K, u2_tempsqr, u2_temp;

  K = K_u1_calc( a1, L, u1 );
  u2_tempsqr = ( K / a2 / a2 + 1.0 ) / L;

  u2_temp = sqrt( u2_tempsqr );

  return u2_temp;
}

double mu_calc(
  double a1,
  double L,
  double u1,
  double R)
{
  double K, mu_temp;

  K = K_u1_calc( a1, L, u1 );

  mu_temp = sqrt( ( R * R + K ) / L ) / R;

  return mu_temp;
}

double hydp( 
  double a1,
  double a2,
  double L,
  double u1,
  double S,
  double c1,
  double c2,
  double pa,
  double pb,
  double R)
{
  double 
    Gauss_pts_minus_point_5[5][5]={
      {0.0, -0.288675134594813, -0.387298334620741,
       -0.430568155797026, -0.453089922969332},
      {0.0, 0.288675134594813, 0.0, -0.169990521792428,
       -0.269234655052841},
      {0.0, 0.0, 0.387298334620741, 0.169990521792428, 0.0},
      {0.0, 0.0, 0.0, 0.430568155797026, 0.269234655052841},
      {0.0, 0.0, 0.0, 0.0, 0.453089922969332} },
    Gauss_wts[5][5]={
      {1.0, 0.5, 0.2777777777778, 0.1739274225687, 0.0},
      {0.0, 0.5, 0.4444444444444, 0.3260725774313, 0.0},
      {0.0, 0.0, 0.2777777777778, 0.3260725774313, 0.0},
      {0.0, 0.0, 0.0, 0.1739274225687, 0.0} ,
       {0.0, 0.0, 0.0, 0.0, 0.0} };
    /* NOTE Gauss weights for 5 pt scheme have not been set */

  int Gauss_scheme, GS_index, i;
  double Rsqr, a1sqr, u1sqr, Lsqr, Ssqr, mu, musqr, K, const1, sum1;
  double Jacobian, sum, xi, Rxi, Rxisqr, muxi, muxisqr;
  double term1, term2, integrand, term3, hydp_temp; 
   
  /* eqn 4.7 Rivlin 1950 */

  Rsqr = R * R;
  a1sqr = a1 * a1;
  u1sqr = u1 * u1;
  Lsqr = L * L;
  Ssqr = S * S;

  mu = mu_calc( a1, L, u1, R );
  musqr = mu * mu;

  term3 = 2.0 * ( c1 / ( Lsqr * musqr ) + 
    c2 * ( 1.0 / Lsqr + 1.0 / musqr + Ssqr * Rsqr ) );

  /* OLD - Now use algebraic integral */
  /* Numerical integral (a1->R) using Gaussian quadrature */
  /* Gauss_scheme = 4;
  GS_index = Gauss_scheme - 1;
  Jacobian = R - a1;
  sum1 = 0.0;
  for(i=0;i<Gauss_scheme;i++)
    {
      xi = 0.5 + Gauss_pts_minus_point_5[i][GS_index];
      Rxi = a1 + xi * Jacobian;
      Rxisqr = Rxi * Rxi;
      muxi = mu_calc( a1, L, u1, Rxi );
      muxisqr = muxi * muxi;
      term1 = 1.0 / ( Lsqr * muxisqr ) - 
	muxisqr * ( 1.0 + Ssqr * Lsqr * Rxisqr );
      term2 = Lsqr * muxisqr - 1.0 / muxisqr;
      integrand = 2.0 * ( c1 * term1 - c2 * term2 ) / 
        ( L * muxisqr * Rxi );
      sum1 = sum1 + Gauss_wts[i][GS_index] * integrand;
    }
  sum1 = sum1 * Jacobian; */

  /* Algebraic integral (a1->R) */
  K = K_u1_calc( a1, L, u1 );
  const1 = c1 / L + c2 * L ;
  sum = const1 * ( 1.0 / ( L * u1sqr ) - Rsqr / ( Rsqr + K ) +
		   log( musqr / u1sqr ) ) - 
    c1 * Ssqr * L * ( Rsqr - a1sqr );

  /*printf( "sum - sum1 = %10.7lf \n", sum - sum1 );*/

  hydp_temp = - sum - term3 + pa;

  return hydp_temp;
}

double trr( 
  double a1,
  double a2,
  double L,
  double u1,
  double S,
  double c1,
  double c2,
  double pa,
  double pb,
  double R)
{
  double mu, musqr, Lsqr, Rsqr, Ssqr;
  double term, p, trr_temp;

  mu = mu_calc( a1, L, u1, R );
  /*printf( "mu  = %10.7lf \n", mu );*/

  musqr = mu * mu;
  Lsqr = L * L;
  Rsqr = R * R;
  Ssqr = S * S;
  
  term = 2.0 * ( c1 / ( Lsqr * musqr ) + 
    c2 * ( 1.0 / Lsqr + 1.0 / musqr + Ssqr * Rsqr ) );
  /*printf( "trr'  = %10.7lf \n", term );*/

  p = hydp( a1, a2, L, u1, S, c1, c2, pa, pb, R );
  /*printf( "p  = %10.7lf \n", p );*/

  trr_temp = term + p;

  return trr_temp;
}

int Solve( 
  double a1,
  double a2,
  double d,
  double L,
  double S,
  double c1,
  double c2,
  double pa,
  double pb,
  double *u1)
{
  int error=0;
  int NMAX = 10;
  double TOL = 1.0e-6;
  double DELTA = 1.0e-4;

  int n;
  double u1_min, udiff, u1a, funca, funcb, dfunc;
  double r1, u2, r2;

  /* Solve trr(r2)+pb=0 for u1 using Newton-Raphson */
  u1a = 1.0;
  udiff = u1a;
  n = 0;
  while ( fabs( udiff / u1a ) > TOL && n < NMAX)
  {
    n++;
    /*printf( "n = %d\n", n );*/

    funca = trr( a1, a2, L, u1a, S, c1, c2, pa, pb, a2 ) + pb;
    /*printf( "funca = %10.7lf\n", funca );*/
    funcb = trr( a1, a2, L, u1a + DELTA, S, c1, c2, pa, pb, a2 ) + pb;
    /*printf( "funcb = %10.7lf\n", funcb );*/
    dfunc = ( funcb - funca ) / DELTA;
    udiff = - funca / dfunc;
    u1a = u1a + udiff;

    /*printf( "dfunc = %10.7lf\n", dfunc );
    printf( "udiff = %10.7lf\n", udiff );
    printf( "u1a = %10.7lf\n", u1a );*/
  }
  if( NMAX == n )
  {
    error = 1;
    printf("Maximum number of iterations exceeded (NMAX=%d)\n",
      NMAX);
  }
  else
  {
    *u1 = u1a;
    r1 = *u1 * a1;
    u2 = u2_calc( a1, a2, L, *u1 );
    r2 = u2 * a2;
    printf( "Deformed radii:\n" );
    printf( "Internal: %10.7lf  External: %10.7lf \n <BR>\n", 
	    r2 / CENTI, r1 / CENTI );
  }
  return error;
}

void CalculateSolution( 
  double a1,
  double a2,
  double d,
  double L,
  double S,
  double c1,
  double c2,
  double pa,
  double pb,
  double u1,
  double R,
  double *p,
  double *EG_tensor,
  double *EG_physical,
  double *TC_tensor,
  double *TC_physical)
{
  int i;
  double mu, musqr, Lsqr, Rsqr, Ssqr;
  double Gxl[6], gzl[6];

  mu = mu_calc( a1, L, u1, R );
  /*printf( "mu = %10.7lf\n", mu );*/

  musqr = mu * mu;
  Lsqr = L * L;
  Rsqr = R * R;
  Ssqr = S * S;
  
  Gxl[0] = 1.0 ;
  Gxl[1] = Rsqr;
  Gxl[2] = 1.0;
  Gxl[3] = 0.0;
  Gxl[4] = 0.0;
  Gxl[5] = 0.0;
  /*printf( "Undeformed covariant metric tensor\n" );
  printf( "%10.7lf %10.7lf %10.7lf \n",   Gxl[0], Gxl[3], Gxl[4] );
  printf( "%10.7lf %10.7lf %10.7lf \n",   Gxl[3], Gxl[1], Gxl[5] );
  printf( "%10.7lf %10.7lf %10.7lf \n\n", Gxl[4], Gxl[5], Gxl[2] );*/

  gzl[0] = 1.0 / ( Lsqr * musqr );
  gzl[1] = musqr * Rsqr;
  gzl[2] = Lsqr + musqr * Rsqr * Lsqr * Ssqr;
  gzl[3] = 0.0;
  gzl[4] = 0.0;
  gzl[5] = musqr * Rsqr * L * S;
  /*printf( "Deformed covariant metric tensor\n" );
  printf( "%10.7lf %10.7lf %10.7lf \n",   gzl[0], gzl[3], gzl[4] );
  printf( "%10.7lf %10.7lf %10.7lf \n",   gzl[3], gzl[1], gzl[5] );
  printf( "%10.7lf %10.7lf %10.7lf \n\n", gzl[4], gzl[5], gzl[2] );*/

  /* Compute Green's strain tensor */
  for(i=0;i<6;i++)
    {
      EG_tensor[i] = ( gzl[i] - Gxl[i] ) / 2.0;
    }
    
  /* Compute physical components of Green's strain */
  EG_physical[0] = EG_tensor[0];
  EG_physical[1] = EG_tensor[1] * musqr * Rsqr;
  EG_physical[2] = EG_tensor[2];
  EG_physical[3] = EG_tensor[3];
  EG_physical[4] = EG_tensor[4];
  EG_physical[5] = EG_tensor[5];

  /* Compute hydrostatic pressure */
  *p = hydp( a1, a2, L, u1, S, c1, c2, pa, pb, R );

  /* Compute physical components of Cauchy stress (Rivlin eqn 4.3) */
  TC_physical[0] = trr( a1, a2, L, u1, S, c1, c2, pa, pb, R );
  TC_physical[1] = 2.0 * ( musqr * ( 1.0 + Ssqr * Lsqr * Rsqr ) * c1 + 
    ( 1.0 / Lsqr + Lsqr * musqr + Ssqr * Rsqr ) * c2 ) + *p;
  TC_physical[2] = 2.0 * ( Lsqr * c1  + 
    ( 1.0 / musqr + Lsqr * musqr ) * c2 ) + *p;
  TC_physical[3] = 0.0;
  TC_physical[4] = 0.0;
  TC_physical[5] = 2.0 * ( S * Lsqr * mu * R * c1 +   S * R * c2 / mu );
 
  /* Compute components of Cauchy stress tensor */
  TC_tensor[0] = TC_physical[0];
  TC_tensor[1] = TC_physical[1] / ( musqr * Rsqr );
  TC_tensor[2] = TC_physical[2];
  TC_tensor[3] = TC_physical[3];
  TC_tensor[4] = TC_physical[4];
  TC_tensor[5] = TC_physical[5];

}

void PrintSolution(
  double a1,
  double a2,
  double L,
  double u1,
  double Rs_SI,
  double p_SI,
  double *EG_tensor,
  double *EG_physical,
  double *TC_tensor_SI,
  double *TC_physical_SI)
{
  double r1_SI, u2, r2_SI;
  
  printf( "<BR>\n");
  printf( "Green's strain tensor undeformed radius Rs=%6.4lf\n <BR>\n", 
    Rs_SI / CENTI );
  printf( "%10.7lf %10.7lf %10.7lf   \n <BR>\n", EG_tensor[0], 
	  EG_tensor[3], EG_tensor[4] );
  printf( "%10.7lf %10.7lf %10.7lf   \n <BR>\n", EG_tensor[3], 
	  EG_tensor[1], EG_tensor[5] );
  printf( "%10.7lf %10.7lf %10.7lf \n\n <BR>\n", EG_tensor[4], 
	  EG_tensor[5], EG_tensor[2] );

  printf( "<BR>\n");
  printf( "Physical Green's strains at undeformed radius Rs=%6.4lf\n <BR>\n", 
    Rs_SI / CENTI );
  printf( "%10.7lf %10.7lf %10.7lf   \n <BR>\n", EG_physical[0], 
	  EG_physical[3], EG_physical[4] );
  printf( "%10.7lf %10.7lf %10.7lf   \n <BR>\n", EG_physical[3], 
	  EG_physical[1], EG_physical[5] );
  printf( "%10.7lf %10.7lf %10.7lf \n\n <BR>\n", EG_physical[4], 
	  EG_physical[5], EG_physical[2] );

  printf( "<BR>\n");
  printf( "Cauchy stress tensor (kPa) at undeformed radius Rs=%6.4lf\n <BR>\n",
	  Rs_SI / CENTI );
  printf( "%10.7lf %10.7lf %10.7lf   \n <BR>\n", TC_tensor_SI[0] / KILO,
	  TC_tensor_SI[3] / KILO, TC_tensor_SI[4] / KILO );
  printf( "%10.7lf %10.7lf %10.7lf   \n <BR>\n", TC_tensor_SI[3] / KILO,
	  TC_tensor_SI[1] / KILO, TC_tensor_SI[5] / KILO );
  printf( "%10.7lf %10.7lf %10.7lf \n\n <BR>\n", TC_tensor_SI[4] / KILO,
	  TC_tensor_SI[5] / KILO, TC_tensor_SI[2] / KILO );

  printf( "<BR>\n");
  printf( "Physical Cauchy stresses (kPa) at undeformed radius Rs=%6.4lf\n <BR>\n", 
	  Rs_SI / CENTI );
  printf( "%10.7lf %10.7lf %10.7lf   \n <BR>\n", TC_physical_SI[0] / KILO,
	  TC_physical_SI[3] / KILO, TC_physical_SI[4] / KILO );
  printf( "%10.7lf %10.7lf %10.7lf   \n <BR>\n", TC_physical_SI[3] / KILO,
	  TC_physical_SI[1] / KILO, TC_physical_SI[5] / KILO );
  printf( "%10.7lf %10.7lf %10.7lf \n\n <BR>\n", TC_physical_SI[4] / KILO,
	  TC_physical_SI[5] / KILO, TC_physical_SI[2] / KILO );

  printf( "<BR>\n");
  printf( "Hydrostatic pressure:  %10.7lf kPa\n <BR>\n", p_SI / KILO );
  printf( "<HR>\n");
}

/* -- The Program Entry Point -- */

int main( int argc, char *argv[] )
{
  double a1, a2, d, L, V, S, c1, c2, pa, pb, Rs, u1, p;
  double EG_tensor[6], EG_physical[6], TC_tensor[6],  TC_physical[6];
  CGI *cgi;

  printf( CGI_HEADER );
  fflush( stdout );

  cgi = getCGIEnvironment( argc, argv );
  if( cgi == NULL )
    {
      printf( "getCGIEnvironment() failed in main()\n" );
    }
  else
    {
      if( nameExists( cgi, "a1" ) )
	{
	  GetValues( cgi, &a1, &a2, &d, &L, &V, &S, 
		     &c1, &c2, &pa, &pb, &Rs );
	  if( !( CheckValues( a1, a2, d, L, V, c1, c2, pa, pb, Rs ) ) )
	    {
	      PrintValues( a1, a2, d, L, V, c1, c2, pa, pb, Rs );
	      if( !( Solve( a1, a2, d, L, S, c1, c2, pa, pb, &u1 ) ) )
		{
		  CalculateSolution( a1, a2, d, L, S, c1, c2, 
				     pa, pb, u1, Rs, &p, 
				     EG_tensor, EG_physical, 
				     TC_tensor, TC_physical);
		  PrintSolution( a1, a2, L, u1, Rs, p, 
				 EG_tensor, EG_physical, 
				 TC_tensor, TC_physical );
		}
	    }
	}
      else
	{
	  printf( CGI_FORM );
	}
    }
	  
  printf( CGI_FOOTER );

  disposeCGI( cgi );
  fflush( stdout );

  return 0;
}
  


