# This example solves a finite elasticity deformation of a wedge 
# loaded by pressure on one face. The symmetry of the problem 
# likens itself to a square-cornered cylinder with a pressure bc 
# on the 'inside' face
#    
#      y
#      ^
#      |
#    2 |\
#      | \
#      |  \     with unit depth
#    1 |\  \
#      | \  \
#      |  \  \
#      --------------> x  
#         1  2
#
# where the x-face (1,0) -> (2,0) was constrained to slide along the
# x-axis, the y-face (0,1) -> (0,2) was constrained to slide 
# along y and one z-face is completely fixed
#
#
# This example has been set up for various combinations of basis functions.
# To run problem for a particular combination uncomment the appropriate section
# in the basis and boundary conditions definitions
#
#
#
# Set up local paths
#

# Command updated by fixcom.sh on Wed Aug 23 17:48:57 NZT 2000
# Old command: assign variable OUTPUT_DIR "output" # local directory for problem output
$OUTPUT_DIR = "output" # local directory for problem output

#
#
#
# Set up problem
#
fem
fem define parameters;r;pressure_inflation;example
fem define coordinates;r;3d_Standard_RC;example reg 1
#
# Basis definitions
#
# CubicHermite-2Linear
#fem define bases;r;CubicHermite_2Linear_3x3x3Gauss_ArithMeanSE;example reg 1    ! Geometry
#fem define ;add bases;r;Linear_CubicHermite_3x3Gauss_ArithMeanSE;example reg 1  ! Faces
#fem define ;add bases;r;5Auxiliary_3x3x3Gauss;example reg 1                     ! Pressure
#fem define ;add bases;r;CubicHermite_Linear_3x3Gauss_ArithMeanSE;example reg 1  ! Faces
#fem define ;add bases;r;2Linear_3x3Gauss;example reg 1                          ! Faces
# OR
# 2CubicHermite-Linear constant pressure
#fem define bases;r;2CubicHermite-Linear_3x3x3Gauss_ArithMeanSE;example reg 1    ! Geometry
#fem define ;add bases;r;2CubicHermite_3x3Gauss_ArithMeanSE;example reg 1        ! Faces
#fem define ;add bases;r;5Auxiliary_3x3x3Gauss;example reg 1                     ! Pressure
#fem define ;add bases;r;CubicHermite_Linear_3x3Gauss_ArithMeanSE;example reg 1  ! Faces
#fem define ;add bases;r;Linear_CubicHermite_3x3Gauss_ArithMeanSE ;examplereg 1  ! Faces
# OR
# 2CubicHermite-Linear 3linear pressure
#fem define bases;r;2CubicHermite-Linear_3x3x3Gauss_ArithMeanSE;example reg 1    ! Geometry
#fem define ;add bases;r;2CubicHermite_3x3Gauss_ArithMeanSE;example reg 1        ! Faces
#fem define ;add bases;r;3Linear-2PressAuxXi3_3x3x3Gauss;example reg 1           ! Pressure
#fem define ;add bases;r;2Linear_3x3Gauss;example reg 1                          ! Faces
#fem define ;add bases;r;CubicHermite_Linear_3x3Gauss_ArithMeanSE;example reg 1  ! Faces
#fem define ;add bases;r;Linear_CubicHermite_3x3Gauss_ArithMeanSE;example reg 1  ! Faces
# OR 
# 3CubicHermite constant pressure
#fem define bases;r;3CubicHermite_3x3x3Gauss_ArithMeanSE;example reg 1           ! Geometry
#fem define ;add bases;r;2CubicHermite_3x3Gauss_ArithMeanSE;example reg 1        ! Faces
#fem define ;add bases;r;5Auxiliary_3x3x3Gauss;example reg 1                     ! Pressure
# OR
# 3CubicHermite 3linear pressure
fem define bases;r;3CubicHermite_3x3x3Gauss_ArithMeanSE;example reg 1            # Geometry
fem define ;add bases;r;2CubicHermite_3x3Gauss_ArithMeanSE;example reg 1         # Faces
fem define ;add bases;r;3Linear-2PressAuxXi3_3x3x3Gauss;example reg 1            # Pressure
fem define ;add bases;r;2Linear_3x3Gauss;example reg 1                           # Faces
#
# Geometry
#
fem define nodes;r;wedge;example reg 1 
fem define elements;r;wedge;example reg 1
fem define fibres;r;wedge;example reg 1
fem define elements;r;wedge;example fibre reg 1
#
# Method
#
fem define equation;r;Incompressible_Finite_Elasticity;example reg 1
fem define material;r;Orthotropic_Incompressible;example reg 1
fem define active;r;active0_00;example reg 1
#
# Boundary Conditions
#
# CubicHermite-2Linear
#fem define initial;r;pressure_CubicHermite-2Linear;example reg 1            -
# OR                                                                          | BC's on derivs 
# 2CubicHermite-Linear constant pressure                                      | in z dirn
#fem define initial;r;pressure_2CubicHermite-Linear_constpres;example reg 1   | not quite
# OR                                                                          | correct
# 2CubicHermite-Linear 3linear pressure                                       |
#fem define initial;r;pressure_2CubicHermite-Linear_3linpres;example reg 1   -
# OR
# 3CubicHermite constant pressure
#fem define initial;r;pressure_3CubicHermite_constpres;example reg 1
# OR
# 3CubicHermite 3linear pressure
fem define initial;r;pressure_3CubicHermite_3linpres;example reg 1
#
#
#
#Solve 
#
fem define solve;r;newton;example reg 1
set out;steps_inflate on
$NAME1 = "wedge_def_00"
$NAME2 = "_active_0000"
$ITERS = 40
$TOL = 0.001

#
#increase pressure incrementally
#
for $PRESS ( 0..10 ) 
{

  if( $PRESS < 1 )
  {
    $FILENAME = "${NAME1}${PRESS}$NAME2"
    $INCREM = 0.0
  } else
  {
    $FILENAME = "${NAME1}${PRESS}$NAME2"
    $INCREM = 0.1
  }

  list assign 
  fem solve increment $INCREM iter $ITERS error $TOL
  fem export nodes;$FILENAME;$OUTPUT_DIR field as wedge
  fem export elements;$FILENAME;$OUTPUT_DIR field as wedge
  fem define initial;w;$FILENAME;$OUTPUT_DIR
}
set out off

fem define window
fem draw lines
fem draw lines deformed dotted 

