# Example b211c - A tri-cubic Hermite, single element cube, using 
# grid-based FEM to solve the Noble 98 ionic current model.

# Set-up some useful variables
$output = ".";
$format = "binary";
# If the example path is not set, default to current directory
if (! defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}

# Define the problem size
fem define parameters;r;$example/minimal;

# Define the geometry - need a tri-cubic Hermite basis for the geometry
# and a tri-linear for the fibres and grid-based FEM
fem define coordinate;r;$example/emech;
fem define node;r;$example/emech-1x1x1-cubic;
fem define base;r;$example/emech-cubic-linear;
fem define element;r;$example/emech-1x1x1-cubic;

# Need to calculate the nodal derivatives since they are not in the
# ip files
fem update node derivative 1 linear;
fem update node derivative 2 linear;
fem update node derivative 3 linear;

fem group element 1 as ALL_ELEMENTS;

# Define 0 degree fibre orientation
fem define fibre;r;$example/emech-0deg;
fem define element;r;$example/emech-1x1x1-linear fibre;

# Define one grid scheme which is 9x9x9 grid points
fem define grid;r;$example/cell-9x9x9;
fem update grid geometry;
fem update grid metric;
fem group grid external as boundary;
# Set the stimulus site
fem group grid xi1=0 oneoff as stimulus_site;

# Define the model to use - Noble 98
fem define equation;r;$example/n98;
#fem define equation;r;n98_collocation;

# Define the cellular material parameters
fem define cell;r;$example/n98;
fem define material;r;$example/n98 cell;

# Define the continuum material parameters - using a 2:1 conductivity 
# ratio (fibre:(sheet/normal))
fem define material;r;$example/n98;
fem update grid material;

# Define the default no-flux boundary condition
fem define initial;r;$example/n98;

# Define the integrator - implicit Adams
fem define solve;r;$example/implicit_grid_based_fem;

# Export the geometry
fem export node;$output/3dcube as cube;
fem export element;$output/3dcube as cube;

# Initialise the solve
fem solve to 0;

# Grab the membrane potential index
fem inquire cell_variable V return_variables VM_ARRAY,VM_IDX;

# Create a history file to hold the membrane potential at all grid
# points through time
fem open history;$output/cube write variables $VM_ARRAY niqslist $VM_IDX $format;

# The time loop
for $time ( 0..5 ) {
    print "Time = $time\n";
    # Solve the model
    fem solve restart to $time;
    # Write the solution to the history file
    fem write history time $time variables $VM_ARRAY $format;
    # Export the potential field for use in CMGUI
    fem export element;$output/potential_$time field as cube;
    # Export the FieldML version - not to useful at present, but needs
    # to be tested.
    fem export fieldml;$output/potential_$time cellml;
}

# Close the history file
fem close history $format;

# Create a CMISS signal file from the history file
fem evaluate electrode;$output/Vm history $output/cube from grid $VM_ARRAY iy $VM_IDX $format;

# Export the membrane potential at all grid points to a UnEMAP signal
# file
fem define export;r;$example/cube;
fem export signal;$output/Vm electrode signal $output/Vm;

# For testing we compare the generated CMISS signal files
if ($TESTING) {
    fem compare signal;Vm basis both masterfile $example/Vm comparefile Vm $format;
}
