if( !$example ) {
  $example = ".";
}

# This example is designed to go through the whole process of getting a potential
# isosurface within a boundary element torso. This is achieved by putting the torso
# inside a box and then defining a grid over the box. A domain solution is then performed
# at each grid point to get an element based representation of the potential field. Some
# Perl routines are then used to convert this field (after exporting) into a shape function
# which is either zero or one, and an extended potential field where the values outside the
# torso are given approximate values to get a smooth potential field throughout the box.

# Set up the three regions for the example
$Torso = 1;
$Heart = 2;
$Box = 3;

# Set up the two classes for the example
$Dipole = 1;
$Grid = 2;

# Define the problem setup - 3d with 3 regions
fem define para;r;$example/torsotimedep 
fem define regi;r;$example/torsouno     
fem define coor 3,1

# Read in the geometry of the 3 regions
fem define node;r;$example/torsotimedep region $Torso,$Heart  
fem define node;r;$example/volume reg $Box
fem define base;r;$example/volume                 
fem define elem;r;$example/torsotimedep region $Torso,$Heart 
fem define elem;r;$example/volume reg $Box

# Define a finite difference grid over the box that encloses the
# torso. The potential field will be evaluated at these points.
fem define grid;r;$example/45k reg $Box
fem update grid geom reg $Box

# Define a quasistatic Laplace equation in the torso regions
# along with the material parameters and initial conditions etc.
fem define equa;r;$example/torsotimedep region $Torso,$Heart class $Dipole lock
fem define mate;r;$example/torsotimedep region $Torso,$Heart
fem define init;r;$example/torsotimedep region $Torso,$Heart
fem define sour;r;$example/test region $Heart 
fem define coup;r;$example/torsotimedep
fem define solv;r;$example/torsotimedep coupled region $Torso,$Heart

# A grid equation needs to be defined so we have a grid class in
# which to store the potentials. Here a Laplace equation is used for
# this purpose.
fem define equa;r;$example/volume cl $Grid reg $Box

# Solve the torso problem. In this case it has been set up to solve for
# only the time step that we want to export the potentials from.
fem solve coupled region $Torso,$Heart class $Dipole

# Export the geometry of the torso and epicardial surfaces
fem export node;hearttorso as hearttorso
fem export elem;hearttorso as hearttorso

# Export the potential field over the torso and epicardial surfaces
fem export node;field as hearttorso field
fem export elem;field as hearttorso field

# Perform a boundary element domain solution at each of the grid points
# to find the value of the potential field. There are two easy ways to get a
# better, smoother potential isosurface but both take substantially longer.
# They involve:
#   Increasing grid density
#   Using adaptive integration 
# This evaluation provides linear speedup with accurate answers when 
# mulitprocessing (if not it certainly has done in the past).
fem evaluate solution grid element 345 optimise iy 1 potential

# Export the geometry of the box that encloses the torso
fem export node;geom as test reg $Box
fem export elem;geom as test reg $Box

# Export the element based field that contains the potential field evaluated
# at all grid points in the torso. All points outside the torso have been set to
# 0.0mV based of the C(P) coefficient that is calculated by the boundary element
# method.
fem export elem;spote as test field reg $Box class $Grid iy 1 

system( "$example/CreateIsosurface.pl" );

if( !$TESTING ) {
  # It is also often useful to visualise isopotential surfaces
  # within the torso. An isopotential surface is creates by creating an isosurface in
  # the box enclosing the torso and using the torso shape function along with an alpha
  # spectrum to ensure the isosurface is only visible inside the torso volume.

  # Read in the geometry of the box containing the torso
  gfx read node geom
  gfx read elem geom

  # Read in the torso geometry (heart and torso BE regions)
  gfx read node hearttorso
  gfx read elem hearttorso

  # Read in the potential field defined by the boundary elements
  gfx read node field
  gfx read elem field

  # To generate these files you need to have a *.exelem file containing the
  # element based potential field. You then need to run the script "CreateIsosurface.pl"
  # which is located in the example directory. This creates two files, the first is a
  # shape file that hsa either zero or one values depending on whether a point is inside
  # or outside the torso mesh. The second file extends the non-zero potential field within
  # the torso out to the edges of the box to make smoother isosurface boundaries. These
  # commands read in the resulting files after the script has been run on "spote.exelem"
  gfx read elem spoteSHAPE
  gfx read elem spotePOTEN

  # Create a 3d window
  gfx create window

  # Create an alpha spectrum which is used to draw the isosurface only inside the torso.
  # It should be noted here that the "overlay_colour" qualifier is used instead of the
  # "overwrite_colour" and this lets the material properties be used as normal with the 
  # alpha spectrum. We define the alpha spectrum to range from 0 to 1.
  gfx create spectrum default+
  gfx modify spectrum default+ clear overlay_colour
  gfx modify spectrum default+ linear reverse range 0 1 extend_above extend_below alpha colour_range 0 1 component 1

  # Define a field called bob which is the same as the shape field but scaled up 1000
  # times (the shape field is either zero or one).
  gfx define field bob scale field shape scale 1000

  # Define a field called bob2 which clamps the maximum field value to 1 (instead of 1000).
  # This is done to get a sharp cutoff at the boundary of the torso.
  gfx define field bob2 clamp_maximum field bob maximum 1

  # Create the isosurface through the specified iso_value of potential and draw it using the
  # native grid point resolution.
  gfx modify g_element test general clear circle_discretization 6 element_discretization "8*8*8" native_discretization potential
  gfx modify g_element test lines select_on material default selected_material default_selected
  gfx modify g_element test iso_surfaces iso_scalar potential iso_value -1.0 use_elements select_on material default_selected data bob2 spectrum default+ selected_material default_selected render_shaded

  # Create an element group that contains the epicardial surface elements and call it heart.
  gfx create egroup heart add 1..40

  # Draw the potential field on the heart using the spectrum "default" to colour according
  # to the potential values.
  gfx modify g_element heart general clear circle_discretization 6 element_discretization "4*4*4" native_discretization none
  gfx modify g_element heart lines select_on material default selected_material default_selected
  gfx modify g_element heart surfaces select_on material default data potential spectrum default selected_material default_selected render_shaded

  # Set the default spectrum to have a range appropriate for showing all of the potential
  # fields.
  gfx modify spectrum default clear overwrite_colour
  gfx modify spectrum default linear reverse range -4.0 2.0 extend_above extend_below rainbow colour_range 0 1 ambient diffuse component 1

  # Create another material called "default+" which is somewhat transparent (alpha=0.3)
  # so we can draw the potential on the torso surface and still see inside to view the
  # potential isosurface.
  gfx create material default+ alpha 0.3

  # Create an element group that contains all of the elements that make up the torso surface
  # called "torso"
  gfx create egroup torso add 41..344

  # Draw the transparent potential field on the surface of the torso using the
  # default spectrum.
  gfx modify g_element torso general clear circle_discretization 6 element_discretization "4*4*4" native_discretization none
  gfx modify g_element torso lines select_on material default selected_material default_selected
  gfx modify g_element torso surfaces select_on material default+ data potential spectrum default selected_material default_selected render_shaded

  # Use slow transparency in the graphics window so the torso surfaces and the potential
  # isosurface are drawn in the correct order.
  gfx mod win 1 set slow

  gfx
}
