# Many of the 3d torso electrical torso solutions are generated using boundary
# elements. This example illustrates how to draw current lines within a boundary
# element torso. This relies on specifying seed points within the box enclosing the
# torso. These seed points are xi positions. Streamlines are then calculated in the
# forward and reverse directions from these seed points.

# Read in the torso geometry (heart and torso BE regions)
gfx read node hearttorso
gfx read elem hearttorso

# Read in the potential field defined by the boundary elements
gfx read node field
gfx read elem field

# Read in the geometry of the box containing the torso
gfx read node geom
gfx read elem geom

# Read in the potential gradient field. This needs to be a 3d (3 component)
# vector field in order to draw streamlines. The components are dphi/dx,
# dphi/dy, dphi/dz.
gfx read elem GradPhi

# Create a 3d viewing window
gfx create window

# Modify the default spectrum so it uses a colour range appropriate for our
# potential values.
gfx modify spectrum default linear reverse range -6.0 2.0 

# Create a material called default+ which is the same as the default
# material except that it has an alpha value of 0.3.
gfx create material default+ alpha 0.3

# These arrays represent the xi seed points in the box that encloses the torso. The
# first array holds the seed points that create the streamlines below the level of the
# heart and the second array holds the seed points that create the streamlines above
# the heart.
@SeedPointsLower = qw/ 0.5,0.5,0.5 0.42,0.5,0.5 0.35,0.5,0.5 0.33,0.5,0.5 0.31,0.5,0.5 0.295,0.5,0.5 0.28,0.5,0.5 /;
@SeedPointsUpper = qw/ 0.5,0.5,0.7 0.5,0.5,0.75 0.5,0.5,0.8 0.5,0.5,0.85 0.5,0.5,0.9 /;

# Concatenate the seed point arrays into a single array
@SeedPoints = ( @SeedPointsLower, @SeedPointsUpper );

# We deal with reasonably small potentials and that means even smaller potential
# gradients. The streamline tracking code takes step sizes based on vector lengths
# so here the vector gradient field is scaled up 1000 times. This is equivalent to
# a linear scaling of the potential field and therefore does not affect the 
# streamlines. 
gfx define field ScalGradPhi scale field gradphi scale 1000 1000 1000

# Loop over each of the seed points in turn and create streamlines that have
# a rectangular cross section in both the forward and reverse directions from
# each seed point. These streamlines can be drawn much faster using lines instead
# of rectangles, or much slower using ellipses. The quality of the streamlines
# depends on the resolution of the grid points that contain the gradient information.
foreach my $seed ( 1..@SeedPoints ) {
  gfx modify g_element test streamlines xi $SeedPoints[$seed-1] rectangle vector ScalGradPhi length 1000 width 10 material default_selected
  gfx modify g_element test streamlines xi $SeedPoints[$seed-1] rectangle vector ScalGradPhi reverse_track length 1000 width 15 material default_selected
}

# Create an element group that contains the epicardial surface elements and draw
# surfaces over the epicardium that are coloured by the potential field.
gfx create egroup heart add 1..40
gfx modify g_element heart surfaces material default data potential spectrum default

# Create an element group that contains the body surface elements and draw
# surfaces over the torso that are coloured by the potential field. Note that
# this uses the transparent material so the streamlines are visible inside.
gfx create egroup torso add 41..344
gfx modify g_element torso surfaces material default+ data potential spectrum default

# Use slow transparency in the graphics window so the torso surfaces streamlines
# are drawn in the correct order.
gfx modify window 1 set slow_trans

gfx
